/*\
 * This file contains declarations of the ColumnListView's header and item views.
 *
 * Copyright (C) 2000-2005 Colin Munro
\*/

#ifndef __LISTVIEWERPRIVATE_H__
#define __LISTVIEWERPRIVATE_H__

class BScrollBar;
class BPopUpMenu;
class BBitmap;
class BCursor;
class CColumn;

// Class that draws the headers, and allows them to be manipulated
class CColumnHeaderView : public BView
{
public:
	CColumnHeaderView(CColumnListView *owner,BRect frame,char *name,uint32 resizingMode);
	~CColumnHeaderView();

	virtual void Draw(BRect updateRect);
	virtual void MouseDown(BPoint point);
	virtual void MouseMoved(BPoint point,uint32 transit,const BMessage *message);
	virtual void MouseUp(BPoint point);
	virtual void MessageReceived(BMessage *message);
private:
	friend CColumnListView;
	friend CColumnItemView;
	CColumnListView *owner;
	CColumn *selected,*resizecolumn;
	float resizepos;

	BPopUpMenu *menu;
	BCursor *cursor;

	CColumn* FindColumn(BPoint);
	BRect FindColumnRect(CColumn*);
	CColumn* FindResizeColumn(BPoint);
	BBitmap* MakeColumnBitmap(CColumn*);
	void FixColumnLevels(int,int);
	void DoContextMenu(CColumn*,char*);
	void ColumnResized(BRect,CColumn*,float);
	void SelectSort(CColumn*);
};

// Class that draws the actual items, and allows them to be manipulated
class CColumnItemView : public BView
{
public:
	CColumnItemView(CColumnListView *owner,BRect frame,char *name,uint32 resizingMode);
	~CColumnItemView();
	
	virtual void Draw(BRect updateRect);
	virtual void MouseDown(BPoint point);
	virtual void FrameResized(float width,float height);

	virtual void Invalidate(void);
	virtual void Invalidate(BRect rect);
	virtual void Invalidate(CRow *row);
	virtual void Invalidate(CRow *row,int iFieldIdentifier);
	virtual void Invalidate(CColumn *column);
	
	virtual void KeyDown(const char *bytes,int32 numBytes);
private:
	friend CColumnListView;
	friend CColumnHeaderView;

	void DoDrawLoop(BRect,CRow*,float,float*);
	CRow* FindExpander(BPoint,CRow*,float,float*,float*);
	void ColumnResized(BRect,CColumn*,float);
	void DoRowExpansion(CRow*,bool);
	bool GetExpanderXY(CRow*,CRow*,float,float*,float*);
	void RecalculateScroll(void);
	bool FindRowRect(CRow*,CRow*,float*,BRect*);
	CRow* FindRow(BPoint,CRow*,float,float*);

	CColumnListView *owner;
	
	bool bDrawDone;
	int iLevel;
};

#endif // __LISTVIEWERPRIVATE_H__
